'' =================================================================================================
''
''   File....... jm_mcp342x.spin
''   Purpose.... Simplified interface for MCP342x ADC
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 12 MAY 2015
''
'' =================================================================================================


con { fixed io pins }

  RX1  = 31                                                      ' programming / debug port
  TX1  = 30
  
  SDA1 = 29                                                      ' boot eeprom
  SCL1 = 28


con { MCP342x }

  #0, L, H, F                                                    ' address pin states (low, high, float)

  WR_342x = %1101_000_0                                          ' write mode
  RD_342x = %1101_000_1                                          ' read mode
  
  CHAN_0  = %00
  CHAN_1  = %01
  CHAN_2  = %10
  CHAN_3  = %11

  SAMP_12 = %00   
  SAMP_14 = %01   
  SAMP_16 = %10   

  GAIN_1  = %00  
  GAIN_2  = %01  
  GAIN_4  = %10  
  GAIN_8  = %11  

  BIT_RDY = 7
  BIT_C1  = 6
  BIT_C0  = 5
  BIT_OC  = 4
  BIT_S1  = 3
  BIT_S0  = 2
  BIT_G1  = 1
  BIT_G0  = 0

  
obj

  i2c : "jm_i2c"
  

var

  byte  devid                                                    ' i2c device id
  byte  cfgbits                                                  ' configuration register bits


pub start(a1, a0)

'' Start MCP342x object
'' -- uses boot eeprom buss
'' -- a1 and a0 are address pin states: L(ow), H(igh), F(loat)
'' -- example: adc.startx(adc#F, adc#F)

  startx(SCL1, SDA1, a1, a0)


pub startx(sclpin, sdapin, a1, a0) | addr

'' Start MCP342x object
'' -- sclpin and sdapin define i2c bus 
'' -- a1 and a0 are address pin states: L(ow), H(igh), F(loat)
'' -- example: adc.startx(SCL1, SDA1, adc#F, adc#F)

  i2c.setupx(sclpin, sdapin)                                     ' connect to i2c bus 

  devid := WR_342x | (%000 << 1)                                 ' default device id

  addr := (a1 << 2) | a0                                         ' build quarternary address

  case addr
    %%00 : devid := WR_342x | (%000 << 1)                        ' A1 = Low,   A0 = Low
    %%20 : devid := WR_342x | (%001 << 1)                        ' A1 = Float, A0 = Low
    %%10 : devid := WR_342x | (%010 << 1)                        ' A1 = High,  A0 = Low
    %%02 : devid := WR_342x | (%011 << 1)                        ' A1 = Low,   A0 = Float   
    %%01 : devid := WR_342x | (%100 << 1)                        ' A1 = Low,   A0 = High   
    %%21 : devid := WR_342x | (%101 << 1)                        ' A1 = Float, A0 = High   
    %%11 : devid := WR_342x | (%110 << 1)                        ' A1 = High,  A0 = High   
    %%12 : devid := WR_342x | (%111 << 1)                        ' A1 = High,  A0 = Float
    %%22 : devid := WR_342x | (%000 << 1)                        ' A1 = Float, A0 = Float       

  cfgbits := %1000_0000

    
pub set_channel(ch, bits, gain)

'' Set MCP342x for continuous conversion
'' -- ch is 0..3
'' -- bits is 0..2 (for 12-, 14-, or 16-bit sample size)
'' -- gain is 0..3 (for 1x, 2x, 4x, or 8x)
'' -- example: adc.set_channel(ch, adc#SAMP_16, adc#GAIN_1)

  cfgbits := (1 << BIT_RDY) | (ch << BIT_C0) | (1 << BIT_OC) | (bits << BIT_S0) | gain

  write_cfg(cfgbits)


pub write_cfg(cfg)

'' Write configuration register to MCP342x

  i2c.start
  i2c.write(devid)                                               ' write mode
  i2c.write(cfg)
  i2c.stop

  cfgbits := cfg & $FF
  

pub read_raw | raw, cfg

'' Read result from previous conversion
'' -- setup by set_channel() or write() to configuration register
'' -- application must correct sign of long value
''    * sign position based on bits used in reading

  raw := 0

  i2c.start
  i2c.write(devid | 1)                                           ' read mode
  raw.byte[1] := i2c.read(i2c#ACK)                               ' high byte of volts
  raw.byte[0] := i2c.read(i2c#ACK)                               ' low byte of volts
  cfgbits     := i2c.read(i2c#NAK)                               ' configuration bits
  i2c.stop             

  return raw                                                     ' return raw reading


pub ready

'' Returns true when ready bit is clear
'' -- use after read()

  if (cfgbits & (1 << BIT_RDY))
    return false
  else
    return true


pub read_mv | raw, bits, gain

'' Returns result of last reading in millivolts
'' -- 0 if device not ready
'' -- use set_channel() or write() prior to calling

  raw := read_raw                                                ' read adc

  ifnot (cfgbits & (1 << BIT_OC))                                ' if not continous mode
    ifnot (ready)                                                '  and conversion not complete
      return 0                                                   '   return 0
    
  bits := (cfgbits >> BIT_S0) & %11                              ' extract bits setting
  gain := cfgbits & %11                                          ' extract gain setting

  return raw2mv(raw, bits, gain)                                 ' return reading in millivolts
  

pub raw2mv(raw, bits, gain) | mv

'' Convert raw reading from MCP342x to millivots
'' -- corrects sign based on bits used for reading
'' -- raw is reading from MCP342x
'' -- bits is 0..2 (for 12-, 14-, or 16-bit sample size)
'' -- gain is 0..3 (for 1x, 2x, 4x, or 8x)
'' -- example: mv := adc.raw2mv(raw, adc#SAMP_16, adc#GAIN_1)

  if (bits == SAMP_12)
    raw := (raw << 20) ~> 20                                     ' fix sign
    mv := raw ~> gain                                            ' convert to millivolts

  elseif (bits == SAMP_14)
    raw := (raw << 18) ~> 18     
    mv := (raw / 4) ~> gain   

  elseif (bits == SAMP_16)    
    mv := (~~raw / 16) ~> gain
    
  else
    mv := 0

  return mv

  
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}